from .dane import dane  # noqa
from .names import danish_names  # noqa
from .names import female_names  # noqa
from .names import load_names  # noqa
from .names import male_names  # noqa
from .names import muslim_names  # noqa
